#ifndef _FolderTree_CPP
#define _FolderTree_CPP
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <Windows.H>
#include <ShlObj.H>
#include <Stdio.H>
#include <ShlOBJ.H>
#include <Stdlib.H>

#include "../Resources/Resource.H"

#include "NSWFL.H"
#include "Package.H"
#include "Entry.H"
#include "FolderTree.H"

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

BOOL FolderTree::DeleteAllItems(void)
{
	return TreeView_DeleteAllItems(lhTreeHandle);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

BOOL FolderTree::GetTreeItemText(HTREEITEM htItem, char *sOUTText)
{
    TVITEMEX tvItemEx;
    tvItemEx.hItem          = htItem;
    tvItemEx.mask           = TVIF_TEXT;
    tvItemEx.pszText        = sOUTText;
    tvItemEx.cchTextMax     = MAX_PATH;

    return TreeView_GetItem(lhTreeHandle, &tvItemEx);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool FolderTree::SelectPath(const char *sINPath)
{
    bool Found = false;

    char sPartBuff[MAX_PATH + 1];
    char sTreeBuff[MAX_PATH + 1];

    HTREEITEM htItem = NULL;
    HTREEITEM htRoot = NULL;

    int iLength = strlen(sINPath);
    int iRPos = 0;
    int iWPos = 0;

    htRoot = TreeView_GetRoot(lhTreeHandle);
    htRoot = TreeView_GetNextItem(lhTreeHandle, htRoot, TVGN_CHILD);

    htItem = htRoot;

	while(true)
    {
        if(sINPath[iRPos] == '\\' || sINPath[iRPos] == '//' || iRPos == iLength)
        {
            sPartBuff[iWPos] = '\0';

            Found = false;

            htItem = TreeView_GetNextItem(lhTreeHandle, htItem, TVGN_CHILD);
            GetTreeItemText(htItem, sTreeBuff);

			if(_strcmpi(sTreeBuff, sPartBuff) == 0)
            {
                TreeView_Expand(lhTreeHandle, htItem, TVE_EXPAND);
                TreeView_SelectItem(lhTreeHandle, htItem);
                Found = true;
            }
            else{
                while((htItem = TreeView_GetNextItem(lhTreeHandle, htItem, TVGN_NEXT)) != NULL)
                {
                    GetTreeItemText(htItem, sTreeBuff);

					if(_strcmpi(sTreeBuff, sPartBuff) == 0)
                    {
                        TreeView_Expand(lhTreeHandle, htItem, TVE_EXPAND);
                        TreeView_SelectItem(lhTreeHandle, htItem);
                        Found = true;
                        break;
                    }
                }
            }

            if(!Found)
			{
                return false;
			}

            if(iRPos == iLength)
			{
                break;
			}

            iWPos = 0;
        }
        else {
            sPartBuff[iWPos] = sINPath[iRPos];
            iWPos++;
        }

        iRPos++;
    }

    return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

HTREEITEM FolderTree::AddTreeViewItem(HTREEITEM hParent, char *sINText, int iImage, int iSelImage)
{
    TV_INSERTSTRUCT MyTVIS;
    MyTVIS.item.hItem          = hParent;
    MyTVIS.item.pszText        = sINText;
    MyTVIS.item.iImage         = iImage;
    MyTVIS.item.iSelectedImage = iSelImage;
    MyTVIS.item.mask           = TVIF_TEXT|TVIF_SELECTEDIMAGE|TVIF_IMAGE;
    MyTVIS.hParent             = hParent;
    MyTVIS.item.cchTextMax     = strlen(sINText);

    return TreeView_InsertItem(lhTreeHandle, &MyTVIS);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool FolderTree::GetDesktopFolderPath(char *sOUTDesktopPath)
{
    DWORD dwSizeOfBuffer = MAX_PATH;

    if(IsWinNT())
    {
        if(Get_StringRegistryValue(HKEY_CURRENT_USER, SHELLFOLDERS, "Desktop", sOUTDesktopPath, dwSizeOfBuffer))
        {
            return true;
        }
        else return false;
    }
    else{
        if(Get_StringRegistryValue(HKEY_LOCAL_MACHINE, SHELLFOLDERS, "Common Desktop", sOUTDesktopPath, dwSizeOfBuffer))
        {
            return true;
        }
        else return false;
    }

    return false;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool FolderTree::IsAlreadyLoaded(const char *sINFolder)
{
    int iPos = 0;

    while(iPos < iLoadedItems)
    {
        if(_strcmpi(sINFolder, sLoadedList[iPos]) == 0)
        {
            return true;
        }
        iPos++;
    }
    return false;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool FolderTree::InitFolderTree(HWND hTreeHandle, HWND hButtonHandle, HWND hEditBohWnd)
{
	bool bResult = true;
	
	lhTreeHandle = hTreeHandle;
    lhButtonHandle = hButtonHandle;
    lhEditBohWnd = hEditBohWnd;

	DeleteAllItems();

    if(!CreateTreeImageList())
    {
        bResult = false;
    }

    if(!InitAlreadyLoaded())
    {
        bResult = false;
    }

    if(!PopBrowseTree())
    {
        bResult = false;
    }

    return bResult;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool FolderTree::UnInitFolderTree(void)
{
	DeleteAllItems();

	if(!FreeAlreadyLoaded())
    {
        return false;
    }

    return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool FolderTree::InitAlreadyLoaded(void)
{
    if((sLoadedList = (char **) calloc(sizeof(char *), LOAD_TREE_ITEMS_INCREMENT)) == NULL)
    {
        return false;
    }

    iMaxLoadedItems = LOAD_TREE_ITEMS_INCREMENT;

    iLoadedItems = 0;

    return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool FolderTree::FreeAlreadyLoaded(void)
{
    int iPos = 0;

    while(iPos < iLoadedItems)
    {
        free(sLoadedList[iPos]);
        sLoadedList[iPos] = NULL;
        iPos++;
    }

    iLoadedItems = 0;
    iMaxLoadedItems = 0;

    free(sLoadedList);

    sLoadedList = NULL;

    return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool FolderTree::IndexDirectoryForSubItem(HTREEITEM hSubLeaf, const char *sINFolder,
										  bool bIndexTwoDeep, bool bIgnoreAlreadyLoaded)
{
    if(!bIgnoreAlreadyLoaded)
	{
		if(IsAlreadyLoaded(sINFolder))
		{
			return true;
		}
	}

    WIN32_FIND_DATA MyFindData;
    HANDLE fHandle = NULL;

    char sDirTemp[MAX_PATH + 1];

    if(sINFolder[strlen(sINFolder) - 1] != '\\')
	{
        sprintf_s(sDirTemp, sizeof(sDirTemp), "%s\\*.*", sINFolder);
	}
    else sprintf_s(sDirTemp, sizeof(sDirTemp), "%s*.*", sINFolder);

    if((fHandle = FindFirstFile(sDirTemp, &MyFindData)) == INVALID_HANDLE_VALUE)
    {
        return false;
    }

	if(!bIgnoreAlreadyLoaded)
	{
		if(iLoadedItems == iMaxLoadedItems)
		{
			sLoadedList = (char **) realloc(sLoadedList, (iMaxLoadedItems + LOAD_TREE_ITEMS_INCREMENT) * sizeof(char *));
			iMaxLoadedItems = (iMaxLoadedItems + LOAD_TREE_ITEMS_INCREMENT);
		}

		int iMem = strlen(sINFolder) + 1;

		sLoadedList[iLoadedItems] = (char *) calloc(sizeof(char), iMem);
		strcpy_s(sLoadedList[iLoadedItems], iMem, sINFolder);
		iLoadedItems++;
	}

    do{
        if( strcmp(MyFindData.cFileName, ".") && strcmp(MyFindData.cFileName, "..") )
        {
            if(MyFindData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
            {
                HTREEITEM hRootLeaf = AddTreeViewItem(hSubLeaf, MyFindData.cFileName, 8, 9);

                if(bIndexTwoDeep)
                {
                    if(sINFolder[strlen(sINFolder) - 1] != '\\')
					{
                        sprintf_s(sDirTemp, sizeof(sDirTemp), "%s\\%s", sINFolder, MyFindData.cFileName);
					}
                    else sprintf_s(sDirTemp, sizeof(sDirTemp), "%s%s", sINFolder, MyFindData.cFileName);

                    IndexDirectoryForSubItem(hRootLeaf, sDirTemp, false, false);
                }
            }
        }
    } while(FindNextFile(fHandle, &MyFindData));

    FindClose(fHandle);

    return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool FolderTree::PopBrowseTree(void)
{
    HTREEITEM hDesktopLeaf = NULL;
	HTREEITEM hMyComputerLeaf = NULL;
	HTREEITEM hSubLeaf = NULL;

    char sDriveText[5];
    char sPathTemp[MAX_PATH + 1];

    int iDrive = 65;  //Drive A
    int iDriveType = 0;
    int iImage = 0;

    hDesktopLeaf = AddTreeViewItem(TVI_ROOT, "Desktop", 6, 6);
    hMyComputerLeaf = AddTreeViewItem(hDesktopLeaf, "MyComputer", 10, 10);

    while(iDrive < 90) //Drive Z
    {
        sprintf_s(sDriveText, sizeof(sDriveText), "%c:", iDrive);

        if((iDriveType = GetDriveType(sDriveText)) != 1)
        {
            if(iDriveType == 0) iImage = 0; //Unknown Type
            else if(iDriveType == 2) iImage = 1; //Removable Disk
            else if(iDriveType == 3) iImage = 2; //Fixed Disk
            else if(iDriveType == 4) iImage = 3; //Remote Network Drive
            else if(iDriveType == 5) iImage = 4; //Cd-Rom Drive
            else if(iDriveType == 6) iImage = 5; //High Speed RAM Drive

            hSubLeaf = AddTreeViewItem(hMyComputerLeaf, sDriveText, iImage, iImage);

            if(iDriveType == 3)
            {
                IndexDirectoryForSubItem(hSubLeaf, sDriveText, false, false);
            }
        }

        iDrive++;
    }

    if(GetDesktopFolderPath(sPathTemp))
    {
        IndexDirectoryForSubItem(hDesktopLeaf, sPathTemp, true, true);
    }

    TreeView_Expand(lhTreeHandle, hDesktopLeaf, TVE_EXPAND);
    TreeView_Expand(lhTreeHandle, hMyComputerLeaf, TVE_EXPAND);

    return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool FolderTree::CreateTreeImageList(void)
{
    hImageList = ImageList_Create(16, 16, ILC_COLOR24, 10, 1);

    HBITMAP hBitMap = NULL;

    hBitMap = LoadBitmap(ghAppInstance, MAKEINTRESOURCE(IDB_TREE_UNKNOWN));
    ImageList_Add(hImageList, hBitMap, hBitMap);
    DeleteObject(hBitMap);

    hBitMap = LoadBitmap(ghAppInstance, MAKEINTRESOURCE(IDB_TREE_REMOVABLE));
    ImageList_Add(hImageList, hBitMap, hBitMap);
    DeleteObject(hBitMap);

    hBitMap = LoadBitmap(ghAppInstance, MAKEINTRESOURCE(IDB_TREE_FIXED));
    ImageList_Add(hImageList, hBitMap, hBitMap);
    DeleteObject(hBitMap);

    hBitMap = LoadBitmap(ghAppInstance, MAKEINTRESOURCE(IDB_TREE_NETWORK));
    ImageList_Add(hImageList, hBitMap, hBitMap);
    DeleteObject(hBitMap);

    hBitMap = LoadBitmap(ghAppInstance, MAKEINTRESOURCE(IDB_TREE_CDROM));
    ImageList_Add(hImageList, hBitMap, hBitMap);
    DeleteObject(hBitMap);

    hBitMap = LoadBitmap(ghAppInstance, MAKEINTRESOURCE(IDB_TREE_RAMDRIVE));
    ImageList_Add(hImageList, hBitMap, hBitMap);
    DeleteObject(hBitMap);

    hBitMap = LoadBitmap(ghAppInstance, MAKEINTRESOURCE(IDB_TREE_DESKTOP));
    ImageList_Add(hImageList, hBitMap, hBitMap);
    DeleteObject(hBitMap);

    hBitMap = LoadBitmap(ghAppInstance, MAKEINTRESOURCE(IDB_TREE_MYDOCS));
    ImageList_Add(hImageList, hBitMap, hBitMap);
    DeleteObject(hBitMap);

    hBitMap = LoadBitmap(ghAppInstance, MAKEINTRESOURCE(IDB_TREE_CLOSEDFOLDER));
    ImageList_Add(hImageList, hBitMap, hBitMap);
    DeleteObject(hBitMap);

    hBitMap = LoadBitmap(ghAppInstance, MAKEINTRESOURCE(IDB_TREE_OPENFOLDER));
    ImageList_Add(hImageList, hBitMap, hBitMap);
    DeleteObject(hBitMap);

    hBitMap = LoadBitmap(ghAppInstance, MAKEINTRESOURCE(IDB_TREE_MYCOMPUTER));
    ImageList_Add(hImageList, hBitMap, hBitMap);
    DeleteObject(hBitMap);

    TreeView_SetImageList(lhTreeHandle, hImageList, TVSIL_NORMAL);

    return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void FolderTree::SetEditCursorToEnd(void)
{
	int iLength = SendMessage(lhEditBohWnd, WM_GETTEXTLENGTH, 0, 0);

	SendMessage(lhEditBohWnd, EM_SETSEL, (WPARAM)iLength, (LPARAM)iLength);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

BOOL FolderTree::FolderTreeHandler(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
    if(uMsg == WM_NOTIFY)
    {
        NMHDR *lpNMH = (LPNMHDR) lParam;

        if(lpNMH->code == TVN_ITEMEXPANDING || lpNMH->code == TVN_SELCHANGED)
        {
            HTREEITEM hItem = NULL;
            HTREEITEM hParent = NULL;

            char sFullSelectedPath[MAX_PATH + 1];
            char sSubIndexFullPath[MAX_PATH + 1];
            char sItemText[MAX_PATH + 1];
            char sSelectedPath[MAX_PATH + 1];
            char sParent[MAX_PATH + 1];

            int iRPos = 0;
            int iWPos = MAX_PATH;

            if(lpNMH->code == TVN_ITEMEXPANDING) // If the use is expanding an item
            {
                LPNMTREEVIEW pnmtv = (LPNMTREEVIEW) lParam;
                hItem = pnmtv->itemNew.hItem;
                TreeView_SelectItem(lhTreeHandle, hItem);
            }
            else { // The user has selected a new tree item
                hItem = TreeView_GetSelection(lhTreeHandle);
            }

            GetTreeItemText(hItem, sItemText);

            if(_strcmpi(sItemText, "MyComputer") == 0)
            {
                EnableWindow(lhButtonHandle, FALSE);
                return TRUE;
            }

            hParent = hItem;

            // It the desktop itsself selected?
            if(_strcmpi(sItemText, "Desktop") == 0)
            {
                if(GetDesktopFolderPath(sFullSelectedPath))
                {
                    if(sFullSelectedPath[strlen(sFullSelectedPath) - 1] != '\\')
                    {
                        strcat_s(sFullSelectedPath, sizeof(sFullSelectedPath), "\\");
                    }
                    
					Set_Text(lhEditBohWnd, sFullSelectedPath);
					SetEditCursorToEnd();

                    EnableWindow(lhButtonHandle, TRUE);
                }
                return TRUE;
            }

            bool bAppendDesktop = false;

            // Find out it the item that is selected is a child of the desktop.
            // If it is, set bAppendDesktop to true.
            while((hParent = TreeView_GetParent(lhTreeHandle, hParent)))
            {
                GetTreeItemText(hParent, sParent);

                if(_strcmpi(sParent, "MyComputer") == 0)
                {
                    break;
                }

                if(_strcmpi(sParent, "desktop") == 0)
                {
                    bAppendDesktop = true;
                    break;
                }
            }

            // Append the name of the selected item to "sSelectedPath" in reverse
            iRPos = strlen(sItemText) - 1;
            while(iRPos != -1)
            {
                if(sItemText[iRPos] == '\0')
				{
					break;
				}

                sSelectedPath[iWPos--] = sItemText[iRPos--];
            }

            //ErrorMessage(NULL, sSelectedPath);

            hParent = hItem;

            // Loop through parents of the selected item.
            // Appending there names to sSelectedPath in reverse.
            while( (hParent = TreeView_GetParent(lhTreeHandle, hParent)))
            {
                GetTreeItemText(hParent, sItemText);

                if(_strcmpi(sItemText, "Desktop") == 0 || _strcmpi(sItemText, "MyComputer") == 0)
                {
                    break;
                }

                sSelectedPath[iWPos--] = '\\';

                iRPos = strlen(sItemText) -1;
                while(iRPos != -1)
                {
                    if(sItemText[iRPos] == '\0')
					{
						break;
					}

                    sSelectedPath[iWPos--] = sItemText[iRPos--];
                }
            }


            // Mirror "sSelectedPath" to sFullSelectedPath (Reverse)
            // Data from the end of "sSelectedPath" to the begining of "sFullSelectedPath" and backwards.
            iRPos = iWPos + 1;
            iWPos = 0;
            while(iRPos != (MAX_PATH + 1))
            {
                if(sSelectedPath[iRPos] == '\0')
				{
					break;
				}

                sFullSelectedPath[iWPos++] = sSelectedPath[iRPos++];
            }

            //MessageBox(GetActiveWindow(), sFullSelectedPath, "sFullSelectedPath", 0);

            sFullSelectedPath[iWPos] = '\0';

            // Is the item that is selected a child of the desktop?
            // If so append its path to the end of the desktop path.
            if(bAppendDesktop)
            {
                if(GetDesktopFolderPath(sSelectedPath))
                {
                    if(sSelectedPath[strlen(sSelectedPath) - 1] != '\\')
                    {
                        strcat_s(sSelectedPath, sizeof(sSelectedPath), "\\");
                    }

                    strcat_s(sSelectedPath, sizeof(sSelectedPath), sFullSelectedPath); // Append "sSelectedPath" to the desktop path
                    strcpy_s(sSubIndexFullPath, sizeof(sSubIndexFullPath), sSelectedPath); // Save "sSubIndexFullPath" for sub indexing

                    strcat_s(sSelectedPath, sizeof(sSelectedPath), "\\");
                    Set_Text(lhEditBohWnd, sSelectedPath);
					SetEditCursorToEnd();

                    EnableWindow(lhButtonHandle, TRUE);
                }
            }
            else{
                strcpy_s(sSubIndexFullPath, sizeof(sSubIndexFullPath), sFullSelectedPath); // Save "sSubIndexFullPath" for sub indexing

                strcat_s(sFullSelectedPath, sizeof(sFullSelectedPath), "\\");
                Set_Text(lhEditBohWnd, sFullSelectedPath);
				SetEditCursorToEnd();
            }

            IndexDirectoryForSubItem(hItem, sSubIndexFullPath, false, false);

            //---------------------------------------------------------------------------
            // Sub Indexing (Begin)
            // This code it designed to load the directorys into the tree "one level lower"
            // than the user currently has selected. So when (if) they do expand the path
            // its contents are already loaded.

            HTREEITEM hSibling = NULL;

            strcpy_s(sSelectedPath, sizeof(sSelectedPath), sSubIndexFullPath);

            if((hSibling = TreeView_GetChild(lhTreeHandle, hItem)) != NULL)
            {
                GetTreeItemText(hSibling, sItemText);

                strcat_s(sSubIndexFullPath, sizeof(sSubIndexFullPath), "\\");
                strcat_s(sSubIndexFullPath, sizeof(sSubIndexFullPath), sItemText);

                IndexDirectoryForSubItem(hSibling, sSubIndexFullPath, false, false);

                while((hSibling = TreeView_GetNextSibling(lhTreeHandle, hSibling)) != NULL)
                {
                    GetTreeItemText(hSibling, sItemText);

                    strcpy_s(sSubIndexFullPath, sizeof(sSubIndexFullPath), sSelectedPath);
                    strcat_s(sSubIndexFullPath, sizeof(sSubIndexFullPath), "\\");
                    strcat_s(sSubIndexFullPath, sizeof(sSubIndexFullPath), sItemText);

                    IndexDirectoryForSubItem(hSibling, sSubIndexFullPath, false, false);
                }
            }

            // Sub Indexing (End)
            //---------------------------------------------------------------------------

            EnableWindow(lhButtonHandle, TRUE);

            return TRUE;
        }
        return FALSE;
    }
    return FALSE;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif

